#ifndef BOITEAOUTILS_H_INCLUDED
#define BOITEAOUTILS_H_INCLUDED
/*
    Batanav est un jeu de bataille navale très classique.
    Copyright (C) 2005 - 2009-2020 Vial Jean-Luc Annecy
    Contact email: vial.frjl@free.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <vector>
#include <map>
#include <complex> // a cause de "vecteur"
#include <wx/string.h>
#include <wx/wx.h>
//---------------------------------------------------------------------------
// Paramètres
#define PARAM_NB_BOUCLES_MODE_AUTO 100 // Nombre de parties en automatique mode machine/machine
const wxString Param_nb_de_boucles=wxString::Format(wxT("%i"),PARAM_NB_BOUCLES_MODE_AUTO);
//---------------------------------------------------------------------------
typedef enum {inconnue,libre,jouee,interdite,interdite_et_jouee,touchee_et_jouee,occupee,touchee} liste_statuts;
typedef std::complex<double>  vecteur;
class case_du_tableau;
typedef std::vector<std::vector<case_du_tableau> > type_contenu;
//---------------------------------------------------------------------------
// Utilitaires divers
template<class Te> Te suivant(Te m){ return (Te(int(m)+1));};
//---------------------------------------------------------------------------
// Emulation de la fonction RandomRange
// retourne un entier dans l'intervalle [low, High[
int RandomRange_bis(int low,int high);
//---------------------------------------------------------------------------
// la définition d'une fonction utilisant un modèle doit être dans le même fichier que sa déclaration
// Elle n'est compilée, avec le type correspondant, que quand elle est instanciée
template<class T> T brasse(T a_brasser)
{
static bool init(false);
typename T::value_type v;
int nb_elements=a_brasser.size();
if(!init) {srand((unsigned)time(NULL));init=true;}
if (nb_elements)
        {
        for (int i=0;i<nb_elements;i++)
                {
                int tirage=RandomRange_bis(i,nb_elements);
                v=a_brasser[i];
                a_brasser[i]=a_brasser[tirage];
                a_brasser[tirage]=v;
                }
        }
return a_brasser;
}
//---------------------------------------------------------------------------
        void RenommeEnfants(wxWindow* Sujet);
//---------------------------------------------------------------------------
// Traducteur
//---------------------------------------------------------------------------
        typedef enum {fr,ang,esp}langues;
        class Traducteur
{
        typedef std::map<langues,wxString> Dictionnaire;
        std::map<wxString,Dictionnaire>textes;
        void ajoute_une_entree(wxString Tf,wxString Ta,wxString Te);
public:
        Traducteur();
        void langue_demarrage();
        langues& fixe_langue();
        wxString trad(wxString T);
        wxString trad_tout(wxString T);

};
//---------------------------------------------------------------------------
// Paramètres globaux
// Taille et couleur par défaut des fenêtres
        wxColor& Couleur_Fenetres_defaut();
        wxRect& Taille_Fenetres_defaut();
        void FixeFontEtTaille(wxWindow* fen);
        class PalettePerso
{
public:
        PalettePerso();
        wxColour clDkGray,clCream,clAqua,clLtGray,clRed,clBlack,clYellow,clLime;
};
// Conversion taille et postion virtuelle sur une grille 1000 x 1000 en taille réelle sur la fenêtre
        wxPoint FaitReelle_Pos(wxPoint posvirtuelle,wxWindow* fen);
        wxSize FaitReelle_Taille(wxSize Sizevirtuelle,wxWindow* fen);

const wxString LF=wxT("\n");
//---------------------------------------------------------------------------

#endif //BOITEAOUTILS_H_INCLUDED
