#include "ProjetdeBase.h"
#include "Calcule.h"
#include "Dessin.h"
// Assure la compatibilité avec la version 2.8 de wxWidgets. A faire sauter ultérieurement
#if !wxCHECK_VERSION(2,9,0)
    #define wxPENSTYLE_SOLID wxSOLID
    #define wxPENSTYLE_DOT wxDOT
    #define wxPENSTYLE_LONG_DASH wxLONG_DASH
    #define wxBRUSHSTYLE_TRANSPARENT wxTRANSPARENT
    #define wxBRUSHSTYLE_SOLID wxSOLID
#endif
//---------------------------------------------------------------------------
//      Implémentation de l'objet Tgraphique
//---------------------------------------------------------------------------

BEGIN_EVENT_TABLE(Tgraphique, wxWindow)
EVT_PAINT(Tgraphique::OnPaint)
EVT_LEFT_DOWN(Tgraphique::OnButton1)
EVT_RIGHT_DOWN(Tgraphique::OnButton1)
END_EVENT_TABLE()

        void Tgraphique::fixe_parametres(int nb)
        {
        PalettePerso p;
        parametres_internes.nb_cases=nb;
        parametres_internes.guirlande_active=false;
        parametres_internes.echiquier_actif=false;
        parametres_internes.pixels=wxPoint(10000,10000);
        parametres_internes.largeur_bord_echiquier=200;
        parametres_internes.largeur_marge=700;
        parametres_internes.largeur_bord_case=150;
        parametres_internes.bord_ombre=p.clDkGray;
        parametres_internes.bord_eclaire=p.clCream;
        parametres_internes.fond_echiquier=p.clAqua;
        parametres_internes.bord_case_ombre=p.clDkGray;
        parametres_internes.bord_case_eclairee=p.clCream;
        parametres_internes.fond_case=p.clLtGray;
        parametres_internes.marque_interdite=p.clRed;
        parametres_internes.marque_jouee=p.clBlack;
        parametres_internes.lignes=p.clBlack;
        parametres_internes.inconnue=p.clAqua;
        }
        //---------------------------------------------------------------------------
        double Tgraphique::cote_case()const
        {
        int x(parametres_internes.pixels.x),
            largeur_bord(parametres_internes.largeur_bord_echiquier),
            nb_cases(parametres_internes.nb_cases);

        return ((x-largeur_bord*2.0)/nb_cases);
        }
        //---------------------------------------------------------------------------
        wxColor Tgraphique::couleur_de(float valeur)
        {
        PalettePerso p;
        wxColor couleurs[6]={p.clDkGray,p.clLtGray,p.clCream,p.clYellow,p.clLime,p.clRed};
        int val=int(valeur);
        val=(val<0?0:val);
        val=(val>5?5:val);
        return couleurs[val];
        }
        //---------------------------------------------------------------------------
        // retourne la case pointée à la souris dans l'échiquier actif
        wxPoint Tgraphique::case_pointee(wxPoint position_souris)
        {
        double c(cote_case()),l(parametres_internes.largeur_bord_echiquier);
        vecteur position=fait_virtXY(position_souris);
            vecteur marge(parametres_internes.largeur_marge,parametres_internes.largeur_marge);
            position-=marge;
            double rx=((parametres_internes.pixels.x-marge.real()*2.0)/parametres_internes.pixels.x);
            double ry=((parametres_internes.pixels.y-marge.imag()*2.0)/parametres_internes.pixels.y);
            position=vecteur(position.real()/rx,position.imag()/ry);
        vecteur bord(l,l);
        int nb_cases=parametres_internes.nb_cases-1;
        position=(position-bord)/c;
        wxPoint resultat(int(position.real()),int(position.imag()));
        resultat=wxPoint((resultat.x<0?0:resultat.x),(resultat.y<0?0:resultat.y));
        resultat=wxPoint((resultat.x>nb_cases?nb_cases:resultat.x),(resultat.y>nb_cases?nb_cases:resultat.y));
        return resultat;
        }
        //---------------------------------------------------------------------------
        //      position sur l'écran sur le canvas actif--> coordonnées sur l'écran virtuel associé
        vecteur Tgraphique::fait_virtXY(const wxPoint& valeur)
        {
        vecteur position(valeur.x,valeur.y);
        vecteur nbpixels(parametres_internes.pixels.x,parametres_internes.pixels.y);
        vecteur resultat(position.real()/GetSize().GetWidth()*nbpixels.real(),position.imag()/GetSize().GetHeight()*nbpixels.imag());
        return resultat;
        }
        //---------------------------------------------------------------------------
        //      position sur l'écran virtuel --> coordonnées sur le canvas  (wxWindow) actif
        wxPoint Tgraphique::fait_absXY(const vecteur& valeur)
        {
        vecteur resultat;
        wxPoint nbpixels(parametres_internes.pixels);
        double Larg(GetSize().GetWidth()),Haut(GetSize().GetHeight());
        resultat=vecteur(valeur.real()/nbpixels.x*Larg,valeur.imag()/nbpixels.y*Haut);
        return wxPoint(int(resultat.real()+0.5),int(resultat.imag()+0.5));
        }
         //---------------------------------------------------------------------------
        //      position sur l'échiquier virtuel --> coordonnées sur le canvas  (wxWindow) actif
        wxPoint Tgraphique::fait_abs_dans_margeXY(vecteur valeur)
        {
            vecteur marge(parametres_internes.largeur_marge,parametres_internes.largeur_marge);
            double rx=((parametres_internes.pixels.x-parametres_internes.largeur_marge*2.0)/parametres_internes.pixels.x);
            double ry=((parametres_internes.pixels.y-parametres_internes.largeur_marge*2.0)/parametres_internes.pixels.y);
            valeur=vecteur(valeur.real()*rx,valeur.imag()*ry);
            valeur+=marge;
            return fait_absXY(valeur);
        }
//---------------------------------------------------------------------------
        // routines de dessin
        // dessine un carré dont le coin supérieur gauche est à "position"
        void Tgraphique::trace_carre(const vecteur& position, double cote,wxColour couleur,wxDC& dc)
        {vecteur diagonale(cote,cote);
        dc.SetBrush(wxBrush(couleur,wxBRUSHSTYLE_SOLID));
        dc.SetPen(wxPen(parametres_internes.lignes,1,wxPENSTYLE_SOLID));
        wxPoint csg,cid;
        csg=fait_abs_dans_margeXY(position);
        cid=fait_abs_dans_margeXY(position+diagonale);
        dc.DrawRectangle(csg.x,csg.y,cid.x-csg.x,cid.y-csg.y);//rectangle(coinsupgauche.x,coinsupgauche.x,largeur, hauteur)
        }
        //---------------------------------------------------------------------------
        // dessine un rectangle centré sur position centre
        void Tgraphique::trace_rectangle(const vecteur& position_centre, const vecteur& diagonale,wxColour couleur,wxDC& dc)
        {
        dc.SetBrush(wxBrush(couleur,wxBRUSHSTYLE_SOLID));
        dc.SetPen(wxPen(parametres_internes.lignes,1,wxPENSTYLE_SOLID));
        wxPoint csg,cid;
        csg=fait_abs_dans_margeXY(position_centre+diagonale*(-1.0));
        cid=fait_abs_dans_margeXY(position_centre+diagonale);
        dc.DrawRectangle(csg.x,csg.y,cid.x-csg.x,cid.y-csg.y);
        }
        //---------------------------------------------------------------------------
        // Dessine un rectangle qui peut avoir une position quelconque
        void Tgraphique::trace_rectangle_incline(const vecteur& position_centre,const vecteur& diagonale,wxColour couleur,double angle_rad,wxDC& dc)
        {using std::polar;
        dc.SetBrush(wxBrush(couleur,wxBRUSHSTYLE_SOLID));
        dc.SetPen(wxPen(couleur,1,wxPENSTYLE_SOLID));

        wxPoint trace[4];
        vecteur sommet [4];
        vecteur diagonale1(diagonale.real(),diagonale.imag()*(-1.0));
        sommet[0]=position_centre+diagonale*polar(1.0,angle_rad);
        sommet[1]=position_centre+diagonale1*polar(1.0,angle_rad);
        sommet[2]=position_centre+diagonale*polar(1.0,angle_rad+M_PI);
        sommet[3]=position_centre+diagonale1*polar(1.0,angle_rad+M_PI);

        for (int i=0;i<4;i++)
                trace[i]=fait_abs_dans_margeXY(sommet[i]);
        dc.DrawPolygon(4,trace);
        }
        //---------------------------------------------------------------------------
        // Dessine un triangle dans un des quatre quartiers d'un carré (0 en bas)
        void Tgraphique::trace_triangle(const vecteur& position,const double& cote,wxColour couleur,int quartier,wxDC& dc)
        {using std::polar;
        dc.SetBrush(wxBrush(couleur,wxBRUSHSTYLE_SOLID));
        dc.SetPen(wxPen(parametres_internes.lignes,1,wxPENSTYLE_SOLID));
        wxPoint trace[3];
        vecteur sommet [3];
        vecteur cote_tri=vecteur(0.5*cote,0.5*cote);
        sommet[0]=position+cote_tri;
        cote_tri *=polar(1.0,M_PI_2*quartier);
        sommet[1]=sommet[0]+cote_tri;
        sommet[2]=sommet[0]+cote_tri*polar(1.0,M_PI_2);

        for (int i=0;i<3;i++)
                trace[i]=fait_abs_dans_margeXY(sommet[i]);
        dc.DrawPolygon(3,trace);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case_relief(const vecteur& position,double cote,int epaisseur,wxColour fond,wxColour b_ombre,wxColour b_lum,bool relief,wxDC& dc)
        //trace une case en relief ou en creux
        {int bo[4]={1,2,3,0};
        if (relief) {bo[0]=3;bo[1]=0;bo[2]=1;bo[3]=2;};
        wxColour coul[4]={b_ombre,b_ombre,b_lum,b_lum};
        for (int i=0;i<4;i++)
                trace_triangle(position,cote,coul[i],bo[i],dc);
        double cote_fond=cote-double(epaisseur)*2;
        trace_carre(position+vecteur(epaisseur,epaisseur),cote_fond,fond,dc);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_echiquier(wxDC& dc)
        {wxColour bord_ombre=parametres_internes.bord_ombre,
                bord_eclaire=parametres_internes.bord_eclaire,
                fond=parametres_internes.fond_echiquier;
        int cote(parametres_internes.pixels.x),larg_bord(parametres_internes.largeur_bord_echiquier);
        trace_case_relief(vecteur(0,0),cote,larg_bord,fond,bord_ombre,bord_eclaire,false,dc);
        guirlande(true,dc);
        guirlande(false,dc);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case_bateau(const vecteur& position,liste_statuts touche_coule,wxDC& dc)
        {wxColour bord_ombre=parametres_internes.bord_case_ombre;
        wxColour bord_ecl=parametres_internes.bord_case_eclairee;
        wxColour fond=parametres_internes.fond_case;
        int larg_bord(parametres_internes.largeur_bord_case);
        trace_case_relief(position,cote_case(),larg_bord,fond,bord_ombre,bord_ecl,touche_coule==occupee,dc);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case_bateau_occupee(const vecteur& position,wxDC& dc)
        {trace_case_bateau(position,occupee,dc);}
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case_bateau_touche(const vecteur& position,wxDC& dc)
        {trace_case_bateau(position,touchee,dc);}
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case_vide(const vecteur& position,wxDC& dc)
        {
        trace_carre(position,cote_case(),parametres_internes.fond_echiquier,dc);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case_jouee(const vecteur& position,wxDC& dc)
        {double cote(cote_case());
        using std::polar;
        vecteur infdroit(cote*0.4,cote/10), centre(cote/2,cote/2);
        trace_carre(position,cote,parametres_internes.fond_echiquier,dc);
        trace_rectangle(position+centre,infdroit,parametres_internes.marque_jouee,dc);
        infdroit*=polar(1.0,M_PI_2);
        trace_rectangle(position+centre,infdroit,parametres_internes.marque_jouee,dc);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case_interdite(const vecteur& position,wxDC& dc)
        {double cote(cote_case());
        using std::polar;
        vecteur infdroit(cote*0.4,cote/10), centre(cote/2,cote/2);
        trace_carre(position,cote,parametres_internes.fond_echiquier,dc);
        trace_rectangle_incline(position+centre,infdroit,parametres_internes.marque_interdite,M_PI/4,dc);
        trace_rectangle_incline(position+centre,infdroit,parametres_internes.marque_interdite,M_PI*3/4,dc);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case_inconnue(const vecteur& position,wxDC& dc)
        {
        trace_carre(position,cote_case(),parametres_internes.inconnue,dc);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_case(const case_du_tableau& c,wxDC& dc)
        {
        double bordure(parametres_internes.largeur_bord_echiquier),cote(cote_case());
        vecteur position(cote*c.abs(),cote*c.ord()) ,origine(bordure,bordure);
        position+=origine;

        switch (c.lit_statut())
                {
              case occupee:trace_case_bateau(position,c.lit_statut(),dc);break;
              case touchee:trace_case_bateau(position,c.lit_statut(),dc);break;
              case libre:trace_case_vide(position,dc);break;
              case interdite:trace_case_interdite(position,dc);break;
              case jouee:trace_case_jouee(position,dc);break;
              default:trace_case_inconnue(position,dc);
                 }
                //inconnue,libre,jouee,interdite,interdite_et_jouee,occupee,touchee

        }
        //---------------------------------------------------------------------------
        // utilisé par TTableau_personnalite
        void Tgraphique::trace_case_couleur(int x,int y,wxColour c,wxDC& dc)
        {
        double bordure(parametres_internes.largeur_bord_echiquier),cote(cote_case());
        vecteur position(cote*x,cote*y) ,origine(bordure,bordure);
        position+=origine;
        trace_carre(position,cote,c,dc);
        }
        //---------------------------------------------------------------------------
        void Tgraphique::guirlande_on_off(bool gui_on)
                {
                parametres_internes.guirlande_active=gui_on;
                }
        //---------------------------------------------------------------------------
        void Tgraphique::enable_on_off(bool ech_on)
                {
                parametres_internes.echiquier_actif=ech_on;
                Enable(ech_on);
                }
        //---------------------------------------------------------------------------
        void Tgraphique::visible_on_off(bool vis_on)
                {
                parametres_internes.echiquier_visible=vis_on;
                }
        //---------------------------------------------------------------------------
        void Tgraphique::guirlande(bool verticale,wxDC& dc)
        {wxPoint p;
        int marge=parametres_internes.largeur_marge;
        double ratio=((parametres_internes.pixels.x-marge*2.0)/parametres_internes.pixels.x);
        double cote=cote_case()*ratio;
        // calcule toutes les positions en positions absolues sur le canvas de Form1
        //vecteur origine(GetPosition().x,GetPosition().y);
        vecteur origine(0,5);
        vecteur offset=vecteur((verticale?0:marge+cote/2.0),(verticale?marge+cote/2.0:0));
        p=fait_absXY(offset);offset=vecteur(p.x,p.y);
        vecteur decalage=vecteur((verticale?0:1),(verticale?1:0))*cote;
        p=fait_absXY(decalage);decalage=vecteur(p.x,p.y);
        vecteur taille_car=vecteur(dc.GetTextExtent(wxT("a")).GetWidth()*(0.5),dc.GetTextExtent(wxT("a")).GetHeight()*(-0.5));

        origine+=offset+taille_car;
        if (parametres_internes.guirlande_active)
                {vecteur p1;
                for (int i=0;i<parametres_internes.nb_cases;i++)
                        {
                        p1=origine+decalage*double(i);
                        p=wxPoint(int(p1.real()),int(p1.imag()));
                        wxChar c=(verticale?wxChar(int('0'+i)):wxChar(int('a'+i)));
                        wxString s(c);
                        dc.DrawText(s,p.x,p.y);
                        }
                }
        }
        //---------------------------------------------------------------------------
        void Tgraphique::trace_tout(wxDC& dc)
        {
                if (parametres_internes.echiquier_visible)
                {
                trace_echiquier(dc);
                int larg=parametres_internes.nb_cases;
                for (int i=0;i<larg;i++)
                        for (int j=0;j<larg;j++)
                            if (parametres_internes.est_perso)
                                trace_case_couleur(i,j,couleur_de(parametres_internes.pers_contenu->valeur_profile(i,j)+0.5),dc);
                            else
                                trace_case((*parametres_internes.ech_contenu)[i][j],dc);
                }
        }
        //---------------------------------------------------------------------------
        void Tgraphique::resize()
        {
        parametres_internes.nb_cases=taille_echiquier_courant();
        }
        //---------------------------------------------------------------------------
        void Tgraphique::OnPaint(wxPaintEvent& WXUNUSED(event))
        {
        wxPaintDC dc(this);
        if (parametres_internes.echiquier_visible) trace_tout(dc);
}
        //---------------------------------------------------------------------------
        // Constructeur pour echiquiers machines
        Tgraphique::Tgraphique(wxWindow* parent,wxWindowID id,wxPoint pos,wxSize taille,int nombre_de_cases,wxString tip):wxWindow(parent,id,pos,taille,wxFULL_REPAINT_ON_RESIZE)
        {
        fixe_parametres(nombre_de_cases);
        parametres_internes.est_perso=false;
        SetToolTip(tip);
        SetBackgroundColour(Couleur_Fenetres_defaut());
        }
        //---------------------------------------------------------------------------
        // Constructeur pour echiquiers personnalités
        Tgraphique::Tgraphique(wxWindow* parent,wxWindowID id,wxPoint pos,wxSize taille,int nombre_de_cases,Tpersonnalite* cont,wxString tip):wxWindow(parent,id,pos,taille,wxFULL_REPAINT_ON_RESIZE)
        {
        fixe_parametres(nombre_de_cases);
        parametres_internes.pers_contenu=cont;
        parametres_internes.est_perso=true;
        SetToolTip(tip);
        SetBackgroundColour(Couleur_Fenetres_defaut());
        }
//---------------------------------------------------------------------------
        void Tgraphique::teste_graphique(wxDC& dc)
        {
        guirlande(false,dc);
        //trace_echiquier();
        /*tableau* t=&Machine1.lui.son_tableau;
        trace_case(case_du_tableau(0,0,occupee,t));
        trace_case(case_du_tableau(2,2,touchee,t));
        trace_case(case_du_tableau(3,3,libre,t));
        trace_case(case_du_tableau(4,4,jouee,t));
        trace_case(case_du_tableau(5,5,interdite,t));
        */
        }
//---------------------------------------------------------------------------
        void Tgraphique::OnButton1(wxMouseEvent& event)
{       // Gère la pression du bouton souris
        // Cas 1 : Echiquier joueur humain lui, construction de flottille
        // Cas 2 : Echiquier joueur humain adversaire choix de case à jouer
        // Cas 3 : Personnalité joueur machine 2 (humaine en général) lui
        // Cas 4 : Personnalité Machine1 lui
        wxPoint p;
        p=event.GetPosition();
        p=case_pointee(p);
        if (!parametres_internes.est_perso) //Echiquier de jeu
            {
            if (!parametres_internes.est_machine1) // joueur humain
                {
                // Cas 1
                if (parametres_internes.est_lui) // choisit une case lors de la construction de l'échiquier
                // Permet la construction de la flottille par le joueur humain.
                // inactif pendant la partie
                // appelle la routine de construction joueur
                    superviseur::ajoute_case_pointee(p);
                // Cas 2
                else // Choisit un coup à jouer
                    {
                    superviseur::joue_un_coup_humain((*parametres_internes.ech_contenu)[p.x][p.y]);
                    parametres_internes.echiquier_actif=false;
                    }
                }
            }
        else //Echiquier de personnalité
            {
            int incr=(event.LeftDown()?1:-1);
            if (!parametres_internes.est_machine1) // Personnalité Machine 2 (joueur humain en général)
                // Cas 3
                {
                Machine2().lui.personnalite.profile[p.x][p.y]+=incr;
                liste_joueurs()[liste_joueurs().index_de(Machine2().nom)].profile[p.x][p.y]=Machine2().lui.personnalite.profile[p.x][p.y];
                }
            else
                // Cas 4
                {
                Machine1().lui.personnalite.profile[p.x][p.y]+=incr;
                liste_joueurs()[liste_joueurs().index_de(Machine1().nom)].profile[p.x][p.y]=Machine1().lui.personnalite.profile[p.x][p.y];
                }
            }
        Refresh();
}
//---------------------------------------------------------------------------
//      Fin de l'objet Tgraphique
//---------------------------------------------------------------------------
