#ifndef MAINFRAME_H_INCLUDED
#define MAINFRAME_H_INCLUDED
/*
    Batanav est un jeu de bataille navale très classique.
    Copyright (C) 2005 - 2009-2020 Vial Jean-Luc Annecy
    Contact email: vial.frjl@free.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// On ajoute les headers "basiques" de wxWidgets
#include <wx/wx.h>
#include "BoiteAOutils.h"
#include "Dessin.h"
#include <wx/html/helpctrl.h>

class Canvas;
class APropos;
class FichePerso;

//---------------------------------------------------------------------------
// Objets propres au programme batanav
//---------------------------------------------------------------------------
void affiche_message(wxString li);
//---------------------------------------------------------------------------
// Principaux paramètres
//---------------------------------------------------------------------------
#define PARAM_TAILLE_GRILLE_PROFILES 5
#define PARAM_TAILLE_MAX_ADMISSIBLE_GRILLE 12
#define PARAM_TAILLE_MAX_ADMISSIBLE_BATEAUX 10
#define PARAM_DECLIC 70  // entre 0 et 100.  à 100 jouera systématiquement la case la moins entourée
#define PARAM_RENFORCEMENT 4 // Renforce le poids de la grille courante connue de l'adversaire par rapport aux grilles passées
//---------------------------------------------------------------------------
// variables globales
//---------------------------------------------------------------------------
int& taille_echiquier_courant();
int taille_grille_profiles();
const wxString nomforme[]={wxT("traditionnelle"),wxT("tordus"),wxT("fumette")};
const wxString nomtaille[]={wxT("5  cases"),wxT("10 cases"),wxT("12 cases")};
// Noms par défaut
const wxString baMACHINE1=wxT("Machine1");
const wxString baMACHINE2=wxT("Machine2");
//---------------------------------------------------------------------------
        // types de bateaux: droits, ou cases juxtaposées, ou cases pouvant
        // se toucher par un seul sommet
typedef enum {lineaires,tordus,par_les_sommets} types_de_bateaux;
//---------------------------------------------------------------------------
class case_du_tableau; // déclaration anticipée pour permettre la dépendance croisée entre tableau et cases_du_tableau
class bateau;
//---------------------------------------------------------------------------
typedef std::vector<case_du_tableau*> plan_bateau;

class nom_des_bateaux
{
        private:
        std::vector<wxString> items;
        public:
        nom_des_bateaux();
        int longeur(wxString n);
        wxString nom_du(int lon);
};
//---------------------------------------------------------------------------
class tableau
{
public:
        Tgraphique* echiquier;
        bool ecran_actif();
        void garnit(liste_statuts st);
        type_contenu contenu;
        tableau();
        tableau(wxString nom_tab,liste_statuts st);
        void init_tableau(wxString nom_tab,liste_statuts st);
        wxString nom;
        int& largeur_tableau();
        int& hauteur_tableau();
        plan_bateau liste_voisins(bateau* client,types_de_bateaux typ,liste_statuts statut_cherche);
        plan_bateau liste_cases_libres();
        void interdit_cases_autour_du_bateau(bateau* client);
        void inscrit_bateau(const bateau& client);
        void prepare_ecran(bool mach1,bool lui);
        void resize();
};
//---------------------------------------------------------------------------
class case_du_tableau
{
        liste_statuts statut;
        int abs_val, ord_val;
public:
        tableau* proprietaire;
        case_du_tableau();
        case_du_tableau(int x,int y,liste_statuts s,tableau* tableau_concerne);
        void change_statut(liste_statuts nouveau);
        liste_statuts lit_statut()const;
        bool memes_coordonnees(case_du_tableau c);
        int ord()const;
        int abs()const;
        bool case_en_coin();
        bool case_au_bord();
        void clone(case_du_tableau* case_nouvelle);
};
//---------------------------------------------------------------------------
class bateau
{
bool cases_connexes();
public:
        tableau* proprietaire;
        types_de_bateaux& forme();
        plan_bateau plan;
        bateau();
        bateau(tableau* t,int longueur);
        int taille;
        wxString Nom();
        bool coule();
        bool contient_case(case_du_tableau c);
        bool ajoute_case(case_du_tableau* c);
        bool ajoute_case(case_du_tableau* c,liste_statuts statut);
        bool ajoute_case_occupee(case_du_tableau* c);
        bool supprime_case(case_du_tableau* c);
        int mode();
        int nb_cotes_au_bord();
        void clone(bateau* nouveau_bateau);
        bateau operator+(const bateau& b);
        void operator+=(const bateau& b);
};
//---------------------------------------------------------------------------
typedef std::vector<bateau> Tflottille;
//---------------------------------------------------------------------------
class Tpersonnalite
{       wxChar nomformate[15];
public:
        void formate_nom(wxString nom);
        int age,victoires;
        // profile n'est pas un vector pour garder une taille fixe
        //lors de l'enregistrement
        //cela limite l'intérêt du paramétrage "taille_grille_profile
        float profile[PARAM_TAILLE_GRILLE_PROFILES][PARAM_TAILLE_GRILLE_PROFILES]; //note pour [compacite][marginalite]
        void purge();
        wxString extrait_nom();
        float valeur_profile(int comp,int marg);
        explicit Tpersonnalite(wxString nom);
};
//---------------------------------------------------------------------------
class TTableau_personnalite
{
public:
        Tgraphique* echiquier;
        bool ecran_actif();
        void prepare_ecran(bool mach1,bool lui);
        TTableau_personnalite();
};
//---------------------------------------------------------------------------
class  Tliste_joueurs
{       std::vector<Tpersonnalite> contenu;
public:
        void liste_par_defaut();
        void push_back(Tpersonnalite p);
        void clear();
        void erase(int p);
        unsigned int size();
        bool contient(wxString nom);
        int index_de(wxString nom);
        Tpersonnalite& operator[](int i);
        Tliste_joueurs();
};
//---------------------------------------------------------------------------
class TJeu
{
public:
        wxString nom;
        tableau son_tableau;
        Tflottille flottille;
        Tpersonnalite personnalite;
        bool ajoute_valeur_personnalite(int comp,int marg,bool vainqueur);
        bateau quel_bateau(case_du_tableau c);
        int indice_du_bateau_incomplet(case_du_tableau c);
        TJeu(wxString nom_jeu,liste_statuts st);
        void associe_canvas(Tgraphique* Canvas_associe);
        void coup_deponge(liste_statuts st);
        void clone(TJeu* jeu_nouveau);
};
//---------------------------------------------------------------------------
class TJoueur
{       void clone_memoire(TJoueur* joueur_nouveau);
public:
        wxString nom;
        TJeu lui,adversaire;
        struct
                {plan_bateau reserve_de_cases_a_jouer;
                Tflottille flottille_adversaire;
                case_du_tableau case_jouee;
                bool bateau_repere;
                } memoire;
        void coup_deponge();
        void vide_memoire();
        bool affecte_personnalite();
        void clone(TJoueur* joueur_nouveau);
        explicit TJoueur(wxString nom_joueur);
};
//---------------------------------------------------------------------------
class TErreur_batnav
{
public:
        wxString texte_erreur;
        TJoueur* client;
        TErreur_batnav();
        explicit TErreur_batnav(wxString tex_er);
        TErreur_batnav(wxString tex_er,TJoueur* j);
};
//---------------------------------------------------------------------------
class TConseils
{
         private:
         std::vector<wxString> items;
         int Nombre_de_conseils;
         public:
         TConseils();
         wxString suivant();
};

//---------------------------------------------------------------------------
// Les joueurs
        TJoueur& Machine1();
        TJoueur& Machine2();
// liste des joueurs
        Tliste_joueurs& liste_joueurs();
//---------------------------------------------------------------------------
// Définition de la classe "MainFrame"
class MainFrame : public wxFrame
{
public:
// Le constructeur
MainFrame();
// Le destructeur
~MainFrame();
        bool init_done;
        bool item_menu_change;
        // Aide
        wxHtmlHelpController* helpController;
        // Contrôles
        wxPanel* BoutonsRadio;
        static const long idDialogues;
        wxTextCtrl* Dialogues;
        static const long idBoutaleau;
        wxButton* Boutaleau;
        void OnBoutaleau(wxCommandEvent& event);
        static const long idBoutouche;
        wxButton* Boutouche;
        void OnBoutouche(wxCommandEvent& event);
        static const long idBoutcoule;
        wxButton* Boutcoule;
        void OnBoutcoule(wxCommandEvent& event);
        static const long idLabEchap;
        wxStaticText* LabEchap;
        static const long idChoixForme;
        wxRadioBox* ChoixForme;
        void OnChoixForme(wxCommandEvent& event);
        static const long idChoixTaille;
        wxRadioBox* ChoixTaille;
        void OnChoixTaille(wxCommandEvent& event);
        void OnStop(wxCommandEvent& event);

        // Canvas de dessin
        static const long idCanvas1;
        Tgraphique* Canvas1;
        static const long idCanvas2;
        Tgraphique* Canvas2;
        static const long idCanvas3;
        Tgraphique* Canvas3;
        static const long idCanvas4;
        Tgraphique* Canvas4;
        FichePerso* PersoJoueur;
private:
        // Noms par defaut
        wxPoint ALaPlace(int x,int y);
        wxString NomFichier;
        wxString InitDir;
        wxString DonneesDir;
        wxString DonneesVarDir;
        wxString NomComplet();
        bool charge_fichier(wxString nomfich);
        void FileOpen(wxCommandEvent& event);
        void FilesOpen(wxCommandEvent& event);
        void FileSave(wxCommandEvent& event);
        void FileSaveAs(wxCommandEvent& event);
        void OnQuit(wxCommandEvent& event);
        void OnClose(wxCloseEvent& event);
        void OnDisclaimer(wxCommandEvent& event);
        void OnAbout(wxCommandEvent& event);
        void OnHelp(wxCommandEvent& event);
        void OnProprietes(wxCommandEvent& event);
        void OnCalcule(wxCommandEvent& event);
        void Onfr(wxCommandEvent& event);
        void Onen(wxCommandEvent& event);
        void Ones(wxCommandEvent& event);
        void OnCloseAbout(wxCommandEvent& event);
        void OnMachHom(wxCommandEvent& event);
        void OnMachMach(wxCommandEvent& event);
        void OnMachMach100(wxCommandEvent& event);
        void OnFichePerso(wxCommandEvent& event);
        void OnPaint(wxPaintEvent& event);


        void rafraichit();

        // Identificateurs

        static const long id_frame;
        static const long ID_BUTTON1;
        static const long ID_STATICTEXT1;
        static const long idMenuQuit;
        static const long idMenuProprietes;
        static const long idMenuCalcule;
        static const long idMenuStop;
        static const long idMenuPerso;
        static const long idMenuDisclaimer;
        static const long idMenuAbout;
        static const long idMenuHelp;
        static const long idMenuMachHom;
        static const long idMenuMachMach;
        static const long idMenuMachMach100;
        static const long idMenufr;
        static const long idMenuen;
        static const long idMenues;
        static const long idMenuToolbar1;
        static const long ID_DIALOGS_FILES_OPEN;
        static const long ID_DIALOGS_FILE_SAVE;
        static const long ID_DIALOGS_FILE_SAVE_AS;
        static const long idaproposinst;
        static const long idficheperso;
        static const long idOnPaint;
        //identificateurs des différents contrôles

        static const long idPanelGauche;
        static const long idStaticBoxDroit;

        APropos* AProposinst;

        DECLARE_EVENT_TABLE()
};
//---------------------------------------------------------------------------
        // Variable globale retournant un pointeur vers la fenêtre principale
        MainFrame*& firstwin();
//---------------------------------------------------------------------------
#endif // MAINFRAME_H_INCLUDED
